using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using H3UtilityToolkit.Misc;
using System.IO;

namespace Johnson
{
    public partial class Form1 : Form
    {
        #region Instance Data

        private string ErrorMsg;
        private int RecentImagesFirstIndex;
        private int RecentMapInfosFirstIndex;
        private ToolStripMenuItem RecentMapsEmptyTool;
        private ToolStripMenuItem RecentImagesEmptyTool;
        private ToolStripMenuItem RecentMapInfosEmptyTool;

        #endregion

        public Form1()
        {
            InitializeComponent();

            ErrorMsg = string.Empty;
            RecentImagesFirstIndex = 0;
            RecentMapInfosFirstIndex = 0;
            RecentMapsEmptyTool = null;
            RecentImagesEmptyTool = null;
            RecentMapInfosEmptyTool = null;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Preferences.Read();

            // Recent files.

            const string none_w = "(none)";

            // Maps.
            if (Preferences.RecentMaps.Count == 0)
            {
                // Add empty.
                RecentMapsEmptyTool = new ToolStripMenuItem(none_w);
                RecentMapsEmptyTool.Enabled = false;
                recentFilesToolStripMenuItem.DropDownItems.Add(RecentMapsEmptyTool);
            }
            else
            {
                for (int x = 0; x < Preferences.RecentMaps.Count; x++)
                {
                    Preferences.RecentMaps[x].Tool.Click +=
                        new EventHandler(RecentMapTool_Click);

                    recentFilesToolStripMenuItem.DropDownItems.Insert(0,
                        Preferences.RecentMaps[x].Tool);
                }
            }
            recentFilesToolStripMenuItem.DropDownItems.Add(new ToolStripSeparator());

            // Images.
            RecentImagesFirstIndex = recentFilesToolStripMenuItem.DropDownItems.Count;
            if (Preferences.RecentImages.Count == 0)
            {
                // Add empty.
                RecentImagesEmptyTool = new ToolStripMenuItem(none_w);
                RecentImagesEmptyTool.Enabled = false;
                recentFilesToolStripMenuItem.DropDownItems.Add(RecentImagesEmptyTool);
            }
            else
            {
                for (int x = 0; x < Preferences.RecentImages.Count; x++)
                {
                    Preferences.RecentImages[x].Tool.Click +=
                        new EventHandler(RecentImageTool_Click);

                    recentFilesToolStripMenuItem.DropDownItems.Insert(RecentImagesFirstIndex,
                        Preferences.RecentImages[x].Tool);
                }
            }
            recentFilesToolStripMenuItem.DropDownItems.Add(new ToolStripSeparator());

            // MapInfos.
            RecentMapInfosFirstIndex = recentFilesToolStripMenuItem.DropDownItems.Count;
            if (Preferences.RecentMapInfos.Count == 0)
            {
                // Add empty.
                RecentMapInfosEmptyTool = new ToolStripMenuItem(none_w);
                RecentMapInfosEmptyTool.Enabled = false;
                recentFilesToolStripMenuItem.DropDownItems.Add(RecentMapInfosEmptyTool);
            }
            else
            {
                for (int x = 0; x < Preferences.RecentMapInfos.Count; x++)
                {
                    Preferences.RecentMapInfos[x].Tool.Click +=
                        new EventHandler(RecentMapInfoTool_Click);

                    recentFilesToolStripMenuItem.DropDownItems.Insert(RecentMapInfosFirstIndex,
                        Preferences.RecentMapInfos[x].Tool);
                }
            }
        }

        #region Recent File's Click Handers

        private void RecentMapTool_Click(object sender, EventArgs e)
        {
            string fileName = ((sender as ToolStripMenuItem).Tag
                as Preferences.RecentFile).FileName;

            if (File.Exists(fileName)) DoMapForm(fileName);
            else
            {
                MessageBox.Show("File no longer exists!", string.Empty,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void RecentImageTool_Click(object sender, EventArgs e)
        {
            string filePath = ((sender as ToolStripMenuItem).Tag
                as Preferences.RecentFile).FileName;

            if (Directory.Exists(filePath)) DoImgForm(filePath);
            else
            {
                MessageBox.Show("Directory no longer exists!", string.Empty,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void RecentMapInfoTool_Click(object sender, EventArgs e)
        {
            string filePath = ((sender as ToolStripMenuItem).Tag
                as Preferences.RecentFile).FileName;

            if (Directory.Exists(filePath)) DoMapInfoForm(filePath);
            else
            {
                MessageBox.Show("Directory no longer exists!", string.Empty,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        #endregion

        #region Add Recent File

        private void AddRecentMap(string fileName)
        {
            if (RecentMapsEmptyTool != null)
            {
                recentFilesToolStripMenuItem.DropDownItems.Remove(RecentMapsEmptyTool);
                RecentMapsEmptyTool = null;

                // Move insert indices up.
                RecentImagesFirstIndex--;
                RecentMapInfosFirstIndex--;
            }
            else
            {
                // First check if we already have..
                for (int x = 0; x < Preferences.RecentMaps.Count; x++)
                {
                    if (fileName == Preferences.RecentMaps[x].FileName)
                        return;
                }
            }

            // If we're currently at the count limit..
            if (Preferences.RecentMaps.Count == Preferences.MaxRecentFiles)
            {
                // Remove oldest.
                recentFilesToolStripMenuItem.DropDownItems.Remove(Preferences.RecentMaps[0].Tool);
                Preferences.RecentMaps.RemoveAt(0);

                // Move insert indices up.
                RecentImagesFirstIndex--;
                RecentMapInfosFirstIndex--;
            }

            // Our new recent file.
            Preferences.RecentFile rf = new Preferences.RecentFile(1, fileName);

            // Correct tool indices.
            byte num = (byte)(Preferences.RecentMaps.Count + 1);
            for (int x = 0; x < Preferences.RecentMaps.Count; x++)
            {
                Preferences.RecentMaps[x].Index = num;
                num--;
            }

            // Add.
            recentFilesToolStripMenuItem.DropDownItems.Insert(0, rf.Tool);
            Preferences.RecentMaps.Add(rf);

            // Move insert indices down.
            RecentImagesFirstIndex++;
            RecentMapInfosFirstIndex++;
        }

        private void AddRecentImage(string fileName)
        {
            if (RecentImagesEmptyTool != null)
            {
                recentFilesToolStripMenuItem.DropDownItems.Remove(RecentImagesEmptyTool);
                RecentImagesEmptyTool = null;

                // Move insert indices up.
                RecentMapInfosFirstIndex--;
            }
            else
            {
                // First check if we already have..
                for (int x = 0; x < Preferences.RecentImages.Count; x++)
                {
                    if (fileName == Preferences.RecentImages[x].FileName)
                        return;
                }
            }

            // If we're currently at the count limit..
            if (Preferences.RecentImages.Count == Preferences.MaxRecentFiles)
            {
                // Remove oldest.
                recentFilesToolStripMenuItem.DropDownItems.Remove(Preferences.RecentImages[0].Tool);
                Preferences.RecentImages.RemoveAt(0);

                // Move insert indices up.
                RecentMapInfosFirstIndex--;
            }

            // Our new recent file.
            Preferences.RecentFile rf = new Preferences.RecentFile(1, fileName);

            // Correct tool indices.
            byte num = (byte)(Preferences.RecentImages.Count + 1);
            for (int x = 0; x < Preferences.RecentImages.Count; x++)
            {
                Preferences.RecentImages[x].Index = num;
                num--;
            }

            // Add.
            recentFilesToolStripMenuItem.DropDownItems.Insert(RecentImagesFirstIndex, rf.Tool);
            Preferences.RecentImages.Add(rf);

            // Move insert indices down.
            RecentMapInfosFirstIndex++;
        }

        private void AddRecentMapInfo(string fileName)
        {
            if (RecentMapInfosEmptyTool != null)
            {
                recentFilesToolStripMenuItem.DropDownItems.Remove(RecentMapInfosEmptyTool);
                RecentMapInfosEmptyTool = null;
            }
            else
            {
                // First check if we already have..
                for (int x = 0; x < Preferences.RecentMapInfos.Count; x++)
                {
                    if (fileName == Preferences.RecentMapInfos[x].FileName)
                        return;
                }
            }

            // If we're currently at the count limit..
            if (Preferences.RecentMapInfos.Count == Preferences.MaxRecentFiles)
            {
                // Remove oldest.
                recentFilesToolStripMenuItem.DropDownItems.Remove(Preferences.RecentMapInfos[0].Tool);
                Preferences.RecentMapInfos.RemoveAt(0);

                // Move insert indices up.
                // none
            }

            // Our new recent file.
            Preferences.RecentFile rf = new Preferences.RecentFile(1, fileName);

            // Correct tool indices.
            byte num = (byte)(Preferences.RecentMapInfos.Count + 1);
            for (int x = 0; x < Preferences.RecentMapInfos.Count; x++)
            {
                Preferences.RecentMapInfos[x].Index = num;
                num--;
            }

            // Add.
            recentFilesToolStripMenuItem.DropDownItems.Insert(RecentMapInfosFirstIndex, rf.Tool);
            Preferences.RecentMapInfos.Add(rf);

            // Move insert indices down.
            // none
        }

        #endregion

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            Preferences.Write();
        }

        #region File Menu

        #region Open Map File

        private void mapFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog o = new OpenFileDialog();
            o.Title = "Open File Dialog";
            o.Filter = "Halo 3 Map Files (*.map)|*.map|All Files (*.*)|*.*";
            o.Multiselect = true;
            if (o.ShowDialog() == DialogResult.Cancel) return;

            for (int x = 0; x < o.FileNames.Length; x++)
                DoMapForm(o.FileNames[x]);

            DisplayAnyErrors();
        }

        private void DoMapForm(string fileName)
        {
            try
            {
                MapForm f = new MapForm(fileName);
                f.MdiParent = this;
                f.Show();
                f.WindowState = FormWindowState.Maximized;
                AddRecentMap(fileName);
            }
            catch (Exception ex)
            {
                ErrorMsg += string.Format("Error opening file \"{0}\": {1}\r\n\r\n",
                    fileName, ex.Message);
            }
        }

        #endregion

        #region Open Image Directory

        private void bLFImageFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            fbd.ShowNewFolderButton = true;
            fbd.Description = "Please select the folder containg the BLF image files.";
            if (fbd.ShowDialog() == DialogResult.Cancel) return;

            DoImgForm(fbd.SelectedPath);
            DisplayAnyErrors();
        }

        private void DoImgForm(string filePath)
        {
            try
            {
                ImgForm f = new ImgForm(filePath);
                f.MdiParent = this;
                f.Show();
                f.WindowState = FormWindowState.Maximized;
                AddRecentImage(filePath);
            }
            catch (Exception ex)
            {
                ErrorMsg += string.Format("Error reading from directory \"{0}\": {1}\r\n\r\n",
                    filePath, ex.Message);
            }
        }

        #endregion

        #region Open MapInfo Directory

        private void mapInfoFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            fbd.ShowNewFolderButton = true;
            fbd.Description = "Please select the folder containg the mapinfo files.";
            if (fbd.ShowDialog() == DialogResult.Cancel) return;

            DoMapInfoForm(fbd.SelectedPath);
            DisplayAnyErrors();
        }

        private void DoMapInfoForm(string filePath)
        {
            try
            {
                MapInfoForm f = new MapInfoForm(filePath);
                f.MdiParent = this;
                f.Show();
                f.WindowState = FormWindowState.Maximized;
                AddRecentMapInfo(filePath);
            }
            catch (Exception ex)
            {
                ErrorMsg += string.Format("Error reading from directory \"{0}\": {1}\r\n\r\n",
                    filePath, ex.Message);
            }
        }

        #endregion

        private void DisplayAnyErrors()
        {
            if (ErrorMsg.Length != 0)
            {
                MessageBox.Show(ErrorMsg, string.Empty,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                ErrorMsg = string.Empty;
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        #endregion

        #region Windows Menu

        private void windowsToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            int len = this.MdiChildren.Length;

            // True if any children.
            toolStripSeparator2.Visible = (len != 0);

            // True if 2 or more children.
            tileHorizontalToolStripMenuItem.Enabled =
            tileVerticalToolStripMenuItem.Enabled =
            cascadeToolStripMenuItem.Enabled = (len > 1);
        }

        private void tileHorizontalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.TileHorizontal);
        }

        private void tileVerticalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.TileVertical);
        }

        private void cascadeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.Cascade);
        }

        #endregion

        #region Help menu

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Application By Prey\r\n\r\nSpecial Thanks Goes Out To:\r\nIron_Forge, Anthony, shade45, Tural, LuxuriousMeat and -DeToX-",
                string.Empty, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void halo3ResearchThreadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Process.Start("http://forums.halomods.com/viewtopic.php?t=69721");
        }

        #endregion

        private void pluginConverterToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new PluginConverter().ShowDialog();
        }
    }
}