using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Extended;

namespace Johnson
{
    public partial class ImgForm : Form
    {
        public ImgForm(string dir)
        {
            InitializeComponent();
            Text = dir;
            comboBox1.Items.AddRange(Enum.GetNames(typeof(PictureBoxSizeMode)));
            comboBox1.Items.RemoveAt(2); // Remove Autosize..
            comboBox1.SelectedIndex = 0;

            const char char_p = 'P';
            foreach (FileInfo f in new DirectoryInfo(dir).GetFiles("*.blf"))
            {
                BLFImageInfo inf = new BLFImageInfo();
                inf.FileName = f.FullName;

                BitReader br = new BitReader(f.FullName, Endian.Little);
                br.BaseStream.Position = 69; // Head len. + 1

                // Terrible way of doing things, but until more research is done..
                if (br.ReadChar() == char_p) inf.Type = BLFType.PNG;
                else inf.Type = BLFType.JFIF;
                br.Close();

                ListViewItem item = new ListViewItem(Path.GetFileNameWithoutExtension(f.Name));
                item.SubItems.Add(inf.Type.ToString());
                item.Tag = inf;
                lv1.Items.Add(item);
            }

            lv1.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.ColumnContent);
            lv1.AutoResizeColumn(1, ColumnHeaderAutoResizeStyle.HeaderSize);
        }

        private void lv1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lv1.FocusedItem == null)
            {
                NoImage();
            }
            else
            {
                try
                {
                    BLFImageInfo inf = (BLFImageInfo)lv1.FocusedItem.Tag;

                    BitReader br = new BitReader(inf.FileName, Endian.Little);
                    br.BaseStream.Position = 68;

                    MemoryStream ms = new MemoryStream(br.ReadBytes((int)br.BaseStream.Length - 68));
                    pictureBox1.Image = Image.FromStream(ms);
                    button1.Enabled = true;
                    comboBox1.Enabled = true;

                    br.Close();
                }
                catch
                {
                    NoImage();
                }
            }
        }

        private void NoImage()
        {
            pictureBox1.Image = null;
            button1.Enabled = false;
            comboBox1.Enabled = false;
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Seeing as we removed Autosize earlier..
            if (comboBox1.SelectedIndex >= (int)PictureBoxSizeMode.AutoSize)
            {
                pictureBox1.SizeMode = (PictureBoxSizeMode)comboBox1.SelectedIndex + 1;
            }
            else
            {
                pictureBox1.SizeMode = (PictureBoxSizeMode)comboBox1.SelectedIndex;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            BLFImageInfo inf = (BLFImageInfo)lv1.FocusedItem.Tag;

            SaveFileDialog s = new SaveFileDialog();
            s.Title = "Save File Dialog";
            s.FileName = Path.GetFileNameWithoutExtension(inf.FileName) + "_img";
            if (inf.Type == BLFType.PNG) s.Filter = "PNG (*.png)|*.png";
            else if (inf.Type == BLFType.JFIF) s.Filter = "JFIF (*.jfif)|*.jfif";

            if (s.ShowDialog() == DialogResult.Cancel) return;

            BitReader br = new BitReader(inf.FileName, Endian.Little);
            br.BaseStream.Position = 68;

            BitWriter bw = new BitWriter(new FileStream(s.FileName, FileMode.Create, FileAccess.ReadWrite, FileShare.ReadWrite), Endian.Little);
            bw.Write(br.ReadBytes((int)br.BaseStream.Length - 68));
            bw.Close();

            br.Close();
        }

        public struct BLFImageInfo
        {
            public string FileName;
            public BLFType Type;
        }

        public enum BLFType
        {
            PNG,
            JFIF
        }
    }
}