using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Extended;
using H3UtilityToolkit;
using H3UtilityToolkit.Essentials;
using H3UtilityToolkit.PluginStuffs;
using H3UtilityToolkit.Structures;

namespace Johnson
{
    public partial class MapForm : Form
    {
        public Map map;
        private Matg matgTag;
        private Scnr scnrTag;
        private Zone zoneTag;

        public MapForm(string fileName)
        {
            InitializeComponent();
            Text = fileName;

            for (int x = 0; x < tabControl1.TabCount; x++) tabControl1.TabPages[x].Tag = false;
            for (int x = 0; x < tabControl2.TabCount; x++) tabControl2.TabPages[x].Tag = false;
            for (int x = 0; x < tabControl3.TabCount; x++) tabControl3.TabPages[x].Tag = false;

            map = new Map(fileName);
            metaEditorPanel1.EditorBackColor = PluginManager.ReflexiveBaseColour;
            tabControl1_SelectedIndexChanged(null, EventArgs.Empty);
        }

        #region Tab Control Events

        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if ((bool)tabControl1.SelectedTab.Tag == false)
            {
                tabControl1.SelectedTab.Tag = true;
                switch (tabControl1.SelectedIndex)
                {
                    case 0: // Map Info
                        {
                            tabControl2_SelectedIndexChanged(null, EventArgs.Empty);
                            break;
                        }
                    case 1: // Resigner
                        {
                            label6.Text = string.Format("Checksum: {0}", map.Header.Checksum.ToString("X"));
                            break;
                        }
                    case 2: // Other
                        {
                            tabControl3_SelectedIndexChanged(null, EventArgs.Empty);
                            break;
                        }
                }
            }
        }

        private void tabControl3_SelectedIndexChanged(object sender, EventArgs e)
        {
            if ((bool)tabControl3.SelectedTab.Tag == false)
            {
                tabControl3.SelectedTab.Tag = true;
                switch (tabControl3.SelectedIndex)
                {
                    case 0: // Reflexive Translator
                        {
                            break;
                        }
                }
            }
        }

        private void tabControl2_SelectedIndexChanged(object sender, EventArgs e)
        {
            if ((bool)tabControl2.SelectedTab.Tag == false)
            {
                tabControl2.SelectedTab.Tag = true;
                switch (tabControl2.SelectedIndex)
                {
                    case 0: // Header
                        {
                            listView1.Items.AddRange(map.Header.ParseForListView());
                            listView1.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
                            break;
                        }
                    case 1: // Index
                        {
                            listView2.Items.AddRange(map.Index.ParseForListView());
                            listView2.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
                            break;
                        }
                    case 2: // Tags
                        {
                            //comboBox2.Items.AddRange(Enum.GetNames(typeof(SortTagsBy)));
                            comboBox2.Items.AddRange(new object[] { "Tag Classes", "Path Hierarchy", "Object Hierarchy" });
                            treeView1.Nodes.Add(map.Header.InternalName);
                            comboBox2.SelectedIndex = 0;
                            metaEditorPanel1.OnResizeHandler += new EventHandler(map.PluginManager.OnPanelResize);
                            break;
                        }
                    case 3: // Strings
                        {
                            listView3.Items.AddRange(map.Strings.ParseForListView());
                            listView3.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
                            break;
                        }
                    case 4: // Locales
                        {
                            int tagIndx = map.Tags.IndexOf("matg", "globals\\globals");
                            if (tagIndx != -1)
                            {
                                matgTag = new Matg(map, map.Tags.TagsList[tagIndx].Identity);
                                comboBox1.Items.AddRange(Enum.GetNames(typeof(Matg.Locale)));
                                comboBox1.SelectedIndex = 0;
                            }
                            break;
                        }
                    case 5: // Scripts
                        {
                            int tagIndx = map.Tags.FirstIndexOf("scnr");
                            if (tagIndx != -1)
                            {
                                scnrTag = new Scnr(map, map.Tags.TagsList[tagIndx].Identity);
                                scnrTag.LoadScriptTable();
                                listView6.Items.AddRange(scnrTag.ParseForListview());
                                listView6.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
                            }
                            break;
                        }
                    case 6: // Assets
                        {
                            int tagIndx = map.Tags.IndexOf("zone", "there they are all standing in a row");
                            if (tagIndx != -1)
                            {
                                zoneTag = new Zone(map, map.Tags.TagsList[tagIndx].Identity);
                                listView5.Items.AddRange(zoneTag.GetAssetTable());
                                listView5.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
                                // This is the tag paths column, and so we shorten it so it doesn't take
                                // up so much room.
                                listView5.Columns[2].Width = (
                                    listView5.Width - SystemInformation.VerticalScrollBarWidth - 4 -
                                    listView5.Columns[0].Width -
                                    listView5.Columns[1].Width -
                                    listView5.Columns[3].Width -
                                    listView5.Columns[4].Width);

                            }
                            break;
                        }
                }
            }
        }

        private void tabControl5_SelectedIndexChanged(object sender, EventArgs e)
        {
            if ((bool)tabControl5.SelectedTab.Tag == false)
            {
                tabControl5.SelectedTab.Tag = true;
                switch (tabControl5.SelectedIndex)
                {
                    case 0: // Details
                        {
                            Tag tag = map.Tags.SelectedTag;
                            exTextBox9.Text = tag.Identity.ToString("X");
                            exTextBox10.Text = tag.MetaOffset.ToString();
                            exTextBox11.Text = tag.MetaSize.ToString();
                            break;
                        }
                    case 1: // Meta Editor
                        {
                            if (map.Tags.SelectedTag.Type == map.PluginManager.CurrentPlugin.Class)
                            {
                                // Only refresh..
                                map.PluginManager.RefreshControls();
                            }
                            else
                            {
                                // Clear.
                                metaEditorPanel1.ClearControls();                                

                                toolStripLabel1.Text = "Loading...";
                                Application.DoEvents();

                                // Read plug-in..
                                if (map.PluginManager.ReadPlugin(map.Tags.SelectedTag.Type))
                                {
                                    // Load controls if no errors.
                                    metaEditorPanel1.AddControlRange(map.PluginManager.ParseForViewing(metaEditorPanel1.EditorWidth));
                                }

                                toolStripLabel1.Text = "Idle";
                            }
                            break;
                        }
                    case 2: // Scanner
                        {
                            listView7.Items.Clear();
                            listView7.AutoResizeColumns(ColumnHeaderAutoResizeStyle.HeaderSize);
                            break;
                        }
                }
            }
        }

        #endregion

        #region Header

        private void listView1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listView1.FocusedItem != null && checkBox1.Checked)
            {
                Clipboard.SetText(listView1.FocusedItem.SubItems[1].Text);
            }
        }

        #endregion

        #region Index

        private void listView2_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listView2.FocusedItem != null && checkBox2.Checked)
            {
                Clipboard.SetText(listView2.FocusedItem.SubItems[1].Text);
            }
        }

        #endregion

        #region Tags

        private void SetSelectedTag()
        {
            switch (map.Tags.SortTagsBy)
            {
                case SortTagsBy.TagClass:
                    {
                        map.Tags.SelectedIndex = map.Tags.IndexOf(
                            treeView1.SelectedNode.Parent.Text,
                            treeView1.SelectedNode.Text);
                        break;
                    }
                case SortTagsBy.PathHierarchy:
                    {
                        int dotIndx = treeView1.SelectedNode.Text.Length - 5;
                        string tt = treeView1.SelectedNode.Text.Substring(dotIndx + 1);
                        string path = treeView1.SelectedNode.Text.Remove(dotIndx);

                        TreeNode curNode = treeView1.SelectedNode.Parent;
                        while (curNode != treeView1.Nodes[0])
                        {
                            path = path.Insert(0, curNode.Text + '\\');
                            curNode = curNode.Parent;
                        }

                        map.Tags.SelectedIndex = map.Tags.IndexOf(tt, path);
                        break;
                    }
                default: // case SortTagsBy.ObjectHierarchy:
                    {
                        map.Tags.SelectedIndex = map.Tags.IndexOf(
                             treeView1.SelectedNode.Parent.Text,
                             treeView1.SelectedNode.Text);
                        break;
                    }
            }
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (e.Node == null || e.Node == treeView1.Nodes[0] || e.Node.Nodes.Count != 0)
            {
                if (tabControl4.TabPages.Contains(tabPage13))
                    tabControl4.TabPages.Remove(tabPage13);
            }
            else
            {
                SetSelectedTag();

                for (int x = 0; x < tabControl5.TabCount; x++) tabControl5.TabPages[x].Tag = false;
                tabControl5_SelectedIndexChanged(null, EventArgs.Empty);

                if (!tabControl4.TabPages.Contains(tabPage13))
                {
                    tabControl4.TabPages.Insert(0, tabPage13);
                }
                tabControl4.SelectedTab = tabPage13;
            }
        }

        private void checkBox3_CheckedChanged_comboBox2_SelectedIndexChanged(object sender, EventArgs e)
        {
            if ((bool)tabControl2.SelectedTab.Tag == true)
            {
                map.Tags.SortTagsBy = (SortTagsBy)comboBox2.SelectedIndex;
                treeView1.Nodes[0].Nodes.Clear();

                switch (map.Tags.SortTagsBy)
                {
                    case SortTagsBy.TagClass:
                        treeView1.Sorted = true;
                        break;

                    case SortTagsBy.PathHierarchy:
                        treeView1.Sorted = false;
                        break;

                    case SortTagsBy.ObjectHierarchy:
                        treeView1.Sorted = false;
                        break;
                }

                treeView1.Nodes[0].Nodes.AddRange(map.Tags.ParseForTreeview(string.Empty, checkBox3.Checked));
                treeView1.Nodes[0].Expand();

                treeView1.SelectedNode = treeView1.Nodes[0];
                TreeViewEventArgs tve = new TreeViewEventArgs(treeView1.SelectedNode);
                treeView1_AfterSelect(null, tve);
            }
        }

        private void tabPage12_Resize(object sender, EventArgs e)
        {
            // The stupid thing doesn't do this itself..
            tabPage12.Invalidate();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            Tag tag = map.Tags.SelectedTag;

            SaveFileDialog s = new SaveFileDialog();
            s.Title = "Save File Dialog";
            s.FileName = (tag.Path.Length == 0 ? tag.Type : Path.GetFileNameWithoutExtension(tag.Path)) + "_meta";
            s.Filter = "Halo 3 Meta Files (*.h3meta)|*.h3meta";

            if (s.ShowDialog() == DialogResult.Cancel) return;

            // Use Litte Endian as it's quicker, and produces same result.
            BitReader br = new BitReader(map.FileName, Endian.Little);
            br.BaseStream.Position = tag.MetaOffset;

            BitWriter bw = new BitWriter(new FileStream(s.FileName, FileMode.Create, FileAccess.ReadWrite, FileShare.ReadWrite), Endian.Little);
            bw.Write(br.ReadBytes(tag.MetaSize));
            bw.Close();

            br.Close();
        }

        private void exTextBox3_TextChanged(object sender, EventArgs e)
        {
            int a;
            button3.Enabled = int.TryParse(exTextBox3.Text, out a);
        }

        private void button3_Click(object sender, EventArgs e)
        {
            if (map.Tags.TagsList.Length == 0)
            {
                exTextBox4.Text = "-1";
                return;
            }

            int mapOffset = int.Parse(exTextBox3.Text);

            // Find
            int curTagIndx = 0;
            int curDist = mapOffset - map.Tags.TagsList[0].MetaOffset;
            if (curDist < 0) curDist = map.Header.Filesize;

            for (int x = 1; x < map.Tags.TagsList.Length; x++)
            {
                int dist = mapOffset - map.Tags.TagsList[x].MetaOffset;
                if (dist >= 0 && dist < curDist)
                {
                    curTagIndx = x;
                    curDist = dist;
                }
            }

            // Meta Offset
            int mOffset = mapOffset - map.Tags.TagsList[curTagIndx].MetaOffset;
            if (mOffset < 0)
            {
                exTextBox4.Text = "-1";
            }
            else
            {
                exTextBox4.Text = mOffset.ToString();

                // Find in tree
                switch (map.Tags.SortTagsBy)
                {
                    case SortTagsBy.TagClass:
                        {
                            foreach (TreeNode classNode in treeView1.Nodes[0].Nodes)
                            {
                                if (classNode.Text == map.Tags.TagsList[curTagIndx].Type)
                                {
                                    foreach (TreeNode pathNode in classNode.Nodes)
                                    {
                                        if (pathNode.Text == map.Tags.TagsList[curTagIndx].Path)
                                        {
                                            treeView1.SelectedNode = pathNode;
                                            break;
                                        }
                                    }
                                    break;
                                }
                            }
                            break;
                        }
                    case SortTagsBy.PathHierarchy:
                        {
                            string[] hier = map.Tags.TagsList[curTagIndx].Path.Split('\\');
                            hier[hier.Length - 1] += string.Format(".{0}", map.Tags.TagsList[curTagIndx].Type);

                            TreeNode curNode = treeView1.Nodes[0];

                            // Loop through pieces
                            for (int y = 0; y < hier.Length; y++)
                            {
                                // Find corresponding node
                                foreach (TreeNode tn in curNode.Nodes)
                                {
                                    if (tn.Text == hier[y])
                                    {
                                        curNode = tn;
                                        break;
                                    }
                                }
                            }
                            treeView1.SelectedNode = curNode;
                            break;
                        }
                    case SortTagsBy.ObjectHierarchy:
                        {
                            bool found = false;
                            foreach (TreeNode gpNode in treeView1.Nodes[0].Nodes)
                            {
                                foreach (TreeNode pNode in gpNode.Nodes)
                                {
                                    foreach (TreeNode classNode in pNode.Nodes)
                                    {
                                        if (classNode.Text == map.Tags.TagsList[curTagIndx].Type)
                                        {
                                            foreach (TreeNode pathNode in classNode.Nodes)
                                            {
                                                if (pathNode.Text == map.Tags.TagsList[curTagIndx].Path)
                                                {
                                                    treeView1.SelectedNode = pathNode;
                                                    found = true;
                                                    break;
                                                }
                                            }
                                            break;
                                        }
                                    }
                                    if (found) break;
                                }
                                if (found) break;
                            }
                            break;
                        }
                }
            }
        }

        #endregion

        #region Meta Editor

        private void preferencesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new MetaEditorPrefs(this).ShowDialog();
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            toolStripLabel1.Text = "Saving...";
            Application.DoEvents();

            map.PluginManager.Write();

            toolStripLabel1.Text = "Idle";
        }

        #endregion

        #region Scanner

        private void button6_Click(object sender, EventArgs e)
        {
            listView7.Items.Clear();
            BitReader br = new BitReader(map.FileName, Endian.Big);

            #region Get Meta End

            int endOfMeta = map.Header.Filesize;
            int matgIndx = map.Tags.IndexOf("matg", "globals\\globals");
            if (matgIndx != -1)
            {
                br.BaseStream.Position = map.Tags.TagsList[matgIndx].MetaOffset + 452 + 8;
                endOfMeta = br.ReadInt32() + map.Header.LocaleTableAddrMod;
            }

            #endregion

            int curPos;
            bool typeFound;

            for (int x = 0; x < map.Tags.SelectedTag.MetaSize; x += 4)
            {
                curPos = map.Tags.SelectedTag.MetaOffset + x;
                typeFound = false;

                #region Dependencies, LoneIDs

                br.BaseStream.Position = curPos;

                string tt = br.ReadString(4);
                Int64 zeros = br.ReadInt64();
                int id = br.ReadInt32();

                int indx = map.Tags.IndexOf(id);
                if (indx != -1 && zeros == 0)
                {
                    ListViewItem lv = new ListViewItem(x.ToString());
                    lv.SubItems.Add(map.Tags.TagsList[indx].Type == tt ? "D" : "L");
                    lv.SubItems.Add(string.Format("{0} - {1}", map.Tags.TagsList[indx].Type, map.Tags.TagsList[indx].Path));
                    listView7.Items.Add(lv);
                    typeFound = true;
                }

                #endregion

                if (typeFound == false)
                {
                    #region Reflexives

                    br.BaseStream.Position = curPos;

                    int cc = br.ReadInt32();
                    int ptr = br.ReadInt32();
                    int tr = ptr - map.Header.MapMagic;

                    if (cc > 0 &&
                        ptr <= 0 &&
                        tr >= map.Header.VirtSegmentStart && tr < endOfMeta)
                    {
                        ListViewItem lv = new ListViewItem(x.ToString());
                        lv.SubItems.Add("R");
                        lv.SubItems.Add(string.Format("{0} - {1}", cc, tr));
                        listView7.Items.Add(lv);
                        typeFound = true;
                    }

                    #endregion

                    if (typeFound == false)
                    {
                        // Strings..
                    }
                }
            }
            br.Close();
            listView7.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
        }

        #endregion

        #region Strings

        private void exTextBox1_TextChanged(object sender, EventArgs e)
        {
            string filter = exTextBox1.Text;
            listView3.Items.Clear();

            if (filter.Length == 0)
            {
                listView3.Items.AddRange(map.Strings.ParseForListView());
            }
            else
            {
                List<ListViewItem> items = new List<ListViewItem>();
                for (int x = 0; x < map.Strings.StringsList.Length; x++)
                {
                    if (filter != exTextBox1.Text) return;

                    if (map.Strings.StringsList[x].Length == 0 || map.Strings.StringsList[x].IndexOf(filter, StringComparison.OrdinalIgnoreCase) == -1)
                        continue;

                    ListViewItem item = new ListViewItem(map.Strings.StringsList[x]);
                    item.SubItems.Add(map.Strings.OffsetsList[x].ToString());
                    item.SubItems.Add(map.Strings.StringsList[x].Length.ToString());
                    items.Add(item);

                    Application.DoEvents();
                }
                listView3.Items.AddRange(items.ToArray());
            }
            listView3.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            string fileName;
            if (exTextBox1.TextLength == 0) fileName = "strings";
            else fileName = string.Format("strings_{0}", exTextBox1.Text);

            SaveFileDialog s = new SaveFileDialog();
            s.Title = "Save File Dialog";
            s.FileName = fileName;
            s.Filter = "Text Files (*.txt)|*.txt";
            if (s.ShowDialog() == DialogResult.Cancel) return;

            StreamWriter sw = new StreamWriter(s.FileName);
            sw.WriteLine(fileName);
            sw.WriteLine(new string('=', fileName.Length + 2));

            foreach (ListViewItem lv in listView3.Items)
                sw.WriteLine(lv.Text);

            sw.Close();
        }

        #endregion

        #region Locales

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            matgTag.LoadLocaleTable((Matg.Locale)comboBox1.SelectedIndex);
            DisplayLocaleStrings();
            exTextBox5.Text = matgTag.TableCount.ToString();
            exTextBox6.Text = matgTag.TableSize.ToString();
            exTextBox7.Text = matgTag.TableOffset.ToString();
            exTextBox8.Text = matgTag.TableIndexOffset.ToString();
        }

        private void exTextBox2_TextChanged(object sender, EventArgs e)
        {
            DisplayLocaleStrings();
        }

        private void DisplayLocaleStrings()
        {
            if (matgTag == null) return;

            string filter = exTextBox2.Text;
            listView4.Items.Clear();
            listView4_SelectedIndexChanged(null, EventArgs.Empty);

            if (filter.Length == 0)
            {
                listView4.Items.AddRange(matgTag.ParseForListview());
            }
            else
            {
                List<ListViewItem> items = new List<ListViewItem>();
                for (int x = 0; x < matgTag.TableCount; x++)
                {
                    if (filter != exTextBox2.Text) return;

                    if (matgTag.LocaleStrings[x].Length == 0 || matgTag.LocaleStrings[x].IndexOf(filter, StringComparison.OrdinalIgnoreCase) == -1)
                        continue;

                    ListViewItem item = new ListViewItem(matgTag.LocaleStrings[x]);
                    item.SubItems.Add(matgTag.LocaleOffsets[x].ToString());
                    item.SubItems.Add(matgTag.LocaleStrings[x].Length.ToString());
                    item.Tag = x;
                    items.Add(item);

                    Application.DoEvents();
                }
                listView4.Items.AddRange(items.ToArray());
            }

            int num = 5;
            if (listView4.Width > num)
            {
                int div = (listView4.Width - SystemInformation.VerticalScrollBarWidth - 3) / num;
                listView4.Columns[0].Width = div * 3;
                listView4.Columns[1].Width = div;
                listView4.Columns[2].Width = div;
            }
        }

        private void button5_Click(object sender, EventArgs e)
        {
            string fileName;
            if (exTextBox2.TextLength == 0) fileName = comboBox1.SelectedItem.ToString();
            else fileName = string.Format("{0}_{1}", comboBox1.SelectedItem, exTextBox2.Text);

            SaveFileDialog s = new SaveFileDialog();
            s.Title = "Save File Dialog";
            s.FileName = fileName;
            s.Filter = "Text Files (*.txt)|*.txt";
            if (s.ShowDialog() == DialogResult.Cancel) return;

            StreamWriter sw = new StreamWriter(s.FileName);
            sw.WriteLine(fileName);
            sw.WriteLine(new string('=', fileName.Length + 2));

            foreach (ListViewItem lv in listView4.Items)
                sw.WriteLine(lv.Text);

            sw.Close();
        }

        private void listView4_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listView4.FocusedItem == null)
            {
                exTextBox24.Text = string.Empty;
                groupBox7.Enabled = false;
            }
            else
            {
                exTextBox24.Text = listView4.FocusedItem.Text;
                exTextBox24.MaxLength = exTextBox24.TextLength;
                groupBox7.Enabled = true;
            }
        }

        private void button9_Click(object sender, EventArgs e)
        {
            if (matgTag == null) return;

            matgTag.Write(
                (listView4.FocusedItem.Tag == null ? listView4.FocusedItem.Index : (int)listView4.FocusedItem.Tag),
                exTextBox24.Text);

            listView4.FocusedItem.Text = exTextBox24.Text;
        }

        #endregion

        #region Scripts

        #endregion

        #region Assets

        private void listView5_SelectedIndexChanged(object sender, EventArgs e)
        {
            BitReader br = new BitReader(map.FileName, Endian.Big);

            if (listView5.FocusedItem == null ||
                int.Parse(listView5.FocusedItem.SubItems[3].Text) < 0 ||
                int.Parse(listView5.FocusedItem.SubItems[3].Text) > br.BaseStream.Length - 24)
            {
                exTextBox14.Text = string.Empty;
                exTextBox15.Text = string.Empty;

                exTextBox16.Text = string.Empty;
                exTextBox17.Text = string.Empty;
                exTextBox18.Text = string.Empty;
                exTextBox19.Text = string.Empty;
                exTextBox20.Text = string.Empty;
                exTextBox21.Text = string.Empty;
            }
            else
            {
                exTextBox14.Text = listView5.FocusedItem.SubItems[3].Text;
                exTextBox15.Text = listView5.FocusedItem.SubItems[4].Text;

                br.BaseStream.Position = int.Parse(listView5.FocusedItem.SubItems[3].Text);

                exTextBox16.Text = br.ReadInt32().ToString();
                exTextBox17.Text = br.ReadInt32().ToString();
                exTextBox18.Text = br.ReadInt32().ToString();
                exTextBox19.Text = br.ReadInt32().ToString();
                exTextBox20.Text = br.ReadInt32().ToString();
                exTextBox21.Text = br.ReadInt32().ToString();
            }
            br.Close();
        }

        #endregion

        #region Analyser

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            panel1.Controls.Clear();
            panel1.Controls.AddRange(map.Analyser.Analyse(panel1.Width));
        }

        #endregion

        #region Resigner

        private void button4_Click(object sender, EventArgs e)
        {
            map.Resign();
            label6.Text = string.Format("Checksum: {0}", map.Header.Checksum.ToString("X"));
        }

        #endregion

        #region Reflexive Translator

        private void exTextBox13_TextChanged(object sender, EventArgs e)
        {
            int a;
            if (int.TryParse(exTextBox13.Text, out a))
            {
                exTextBox12.Text = (a - map.Header.MapMagic).ToString();
            }
            else
            {
                exTextBox12.Text = string.Empty;
            }
        }

        private void exTextBox22_TextChanged(object sender, EventArgs e)
        {
            int a;
            if (int.TryParse(exTextBox22.Text, out a))
            {
                exTextBox23.Text = (a + map.Header.MapMagic).ToString();
            }
            else
            {
                exTextBox23.Text = string.Empty;
            }
        }

        #endregion
    }
}