using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Extended;

namespace Johnson
{
    public partial class MapInfoForm : Form
    {
        #region Instance Data

        private static string[] UnicodeNames;
        private const int UnicodeNameBlockLength = 64;
        private const int NullNameBlockIndex = 9;

        private static string[] UnicodeDescs;
        private const int UnicodeDescBlockLength = 256;
        private const int NullDescBlockIndex = 9;

        private static string[] InternalNames;

        #endregion

        static MapInfoForm()
        {
            UnicodeNames = new string[] { 
                "English Name", "Japanese Name", "German Name", "French Name", 
                "Spanish Name", "Latin American Spanish Name", "Italian Name", 
                "Korean Name", "Chinese Name", string.Empty, "Portuguese Name" };

            UnicodeDescs = new string[] { 
               "English Desc", "Japanese Desc", "German Desc", "French Desc", 
                "Spanish Desc", "Latin American Spanish Desc", "Italian Desc", 
                "Korean Desc", "Chinese Desc", string.Empty, "Portuguese Desc" };

            InternalNames = new string[] { 
                "Internal Name", "Internal Name" };
        }

        public MapInfoForm(string dir)
        {
            InitializeComponent();
            Text = dir;

            foreach (FileInfo f in new DirectoryInfo(dir).GetFiles("*.mapinfo"))
            {
                ListViewItem item = new ListViewItem(Path.GetFileNameWithoutExtension(f.Name));
                item.Tag = f.FullName;
                lv1.Items.Add(item);
            }

            // Names
            for (int x = 0; x < UnicodeNames.Length; x++)
            {
                if (x != NullNameBlockIndex)
                {
                    ListViewItem item = new ListViewItem(UnicodeNames[x]);
                    item.SubItems.Add(string.Empty);
                    item.BackColor = Color.LightSteelBlue;
                    lv2.Items.Add(item);
                }
            }
            // Descriptions
            for (int x = 0; x < UnicodeDescs.Length; x++)
            {
                if (x != NullDescBlockIndex)
                {
                    ListViewItem item = new ListViewItem(UnicodeDescs[x]);
                    item.SubItems.Add(string.Empty);
                    item.BackColor = Color.Gainsboro;
                    lv2.Items.Add(item);
                }
            }
            // Internal Names
            for (int x = 0; x < InternalNames.Length; x++)
            {
                ListViewItem item = new ListViewItem(InternalNames[x]);
                item.SubItems.Add(string.Empty);
                item.BackColor = Color.Silver;
                lv2.Items.Add(item);
            }

            lv1.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
            lv2.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
        }

        private void lv1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lv1.FocusedItem == null)
            {
                for (int x = 0; x < lv2.Items.Count; x++)
                {
                    lv2.Items[x].SubItems[1].Text = string.Empty;
                }
                lv2.AutoResizeColumn(1, ColumnHeaderAutoResizeStyle.ColumnContent);
            }
            else
            {
                try
                {
                    BitReader br = new BitReader((string)lv1.FocusedItem.Tag, Endian.Little);
                    br.BaseStream.Position = 69;
                    int lvIndex = 0;

                    // Names
                    for (int x = 0; x < UnicodeNames.Length; x++)
                    {
                        if (x == NullNameBlockIndex)
                        {
                            br.BaseStream.Position += UnicodeNameBlockLength;
                        }
                        else
                        {
                            lv2.Items[lvIndex++].SubItems[1].Text = 
                                br.ReadString(UnicodeNameBlockLength, Encoding.Unicode);
                        }
                    }
                    br.BaseStream.Position += 64;

                    // Descriptions
                    for (int x = 0; x < UnicodeDescs.Length; x++)
                    {
                        if (x == NullDescBlockIndex)
                        {
                            br.BaseStream.Position += UnicodeDescBlockLength;
                        }
                        else
                        {
                            lv2.Items[lvIndex++].SubItems[1].Text = 
                                br.ReadString(UnicodeDescBlockLength, Encoding.Unicode);
                        }
                    }
                    br.BaseStream.Position += 255;

                    // Internal Names
                    for (int x = 0; x < InternalNames.Length; x++)
                    {
                        lv2.Items[lvIndex++].SubItems[1].Text = 
                            br.ReadString(256);
                    }

                    lv2.AutoResizeColumn(1, ColumnHeaderAutoResizeStyle.ColumnContent);
                    br.Close();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(string.Format("An error occured whilst parsing the file. \r\n\"{0}\"", ex.Message),
                        string.Empty, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }
    }
}