using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using H3UtilityToolkit;
using H3UtilityToolkit.PluginStuffs;

namespace Johnson
{
    public partial class PluginConverter : Form
    {
        public PluginConverter()
        {
            InitializeComponent();
            this.MaximumSize = this.Size;

            string[] datatypes = Enum.GetNames(typeof(PluginManager.PluginType));
            comboBox4.Items.AddRange(datatypes);
            comboBox4.SelectedIndex = 0;
            comboBox5.Items.AddRange((object[])datatypes.Clone());
            comboBox5.SelectedIndex = 0;
        }

        private void button7_Click(object sender, EventArgs e)
        {
            OpenFileDialog o = new OpenFileDialog();
            o.Title = "Open File Dialog";
            o.InitialDirectory = PluginManager.PluginDirectory;
            o.Filter = "All Files (*.*)|*.*";
            if (o.ShowDialog() == DialogResult.Cancel) return;
            exTextBox25.Text = o.FileName;
        }

        private void exTextBox25_TextChanged(object sender, EventArgs e)
        {
            if (File.Exists(exTextBox25.Text))
            {
                button8.Enabled = true;
                switch (Path.GetExtension(exTextBox25.Text))
                {
                    case ".dta":
                        exTextBox27.Text = "DTA";
                        break;

                    case ".ent":
                        exTextBox27.Text = "ENT";
                        break;

                    case ".cs":
                        exTextBox27.Text = "CS";
                        break;

                    default:
                        exTextBox27.Text = "UNK";
                        button8.Enabled = false;
                        break;
                }
            }
            else
            {
                button8.Enabled = false;
            }
        }

        private void button8_Click(object sender, EventArgs e)
        {
            SaveFileDialog s = new SaveFileDialog();
            s.Title = "Save File Dialog";
            s.FileName = Path.GetFileNameWithoutExtension(exTextBox25.Text);
            switch (comboBox4.SelectedIndex)
            {
                case 0:
                    s.Filter = "DTA Files (*.dta)|*.dta";
                    break;

                case 1:
                    s.Filter = "ENT Files (*.ent)|*.ent";
                    break;

                case 2:
                    s.Filter = "CS Files (*.cs)|*.cs";
                    break;
            }
            if (s.ShowDialog() == DialogResult.Cancel) return;

            progressBar1.Value = 0;
            progressBar1.Maximum = 1;
            progressBar1.Step = 1;
            progressBar1.PerformStep();

            PluginManager.ConvertPlugin(
                exTextBox25.Text,
                (PluginManager.PluginType)Enum.Parse(typeof(PluginManager.PluginType), exTextBox27.Text),
                s.FileName,
                (PluginManager.PluginType)Enum.Parse(typeof(PluginManager.PluginType), comboBox4.SelectedItem.ToString())
            );

            progressBar1.Value = 0;
        }

        private void button11_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog f = new FolderBrowserDialog();
            f.Description = "Please select the folder containing the plugins you wish to convert.";
            f.SelectedPath = PluginManager.PluginDirectory;
            if (f.ShowDialog() == DialogResult.Cancel) return;
            exTextBox26.Text = f.SelectedPath;
        }

        private void exTextBox26_TextChanged(object sender, EventArgs e)
        {
            if (Directory.Exists(exTextBox26.Text))
            {
                button10.Enabled = true;
                string fileType = string.Empty;
                foreach (FileInfo inf in new DirectoryInfo(exTextBox26.Text).GetFiles())
                {
                    switch (inf.Extension)
                    {
                        case ".dta":
                            fileType = "DTA";
                            break;

                        case ".ent":
                            fileType = "ENT";
                            break;

                        case ".cs":
                            fileType = "CS";
                            break;

                        default:
                            fileType = string.Empty;
                            break;
                    }
                    if (fileType != string.Empty && !exTextBox28.Text.Contains(fileType))
                    {
                        exTextBox28.Text +=
                            (exTextBox28.TextLength == 0 ? string.Empty : ", ") + fileType;
                    }
                }
            }
            else
            {
                button10.Enabled = false;
            }
        }

        private void button10_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog f = new FolderBrowserDialog();
            f.Description = "Please select the folder in which you want the new plugins to be saved.";
            f.SelectedPath = PluginManager.PluginDirectory;
            if (f.ShowDialog() == DialogResult.Cancel) return;

            string outputExt = comboBox5.SelectedItem.ToString().ToLower();
            PluginManager.PluginType outputType =
                (PluginManager.PluginType)Enum.Parse(typeof(PluginManager.PluginType), comboBox5.SelectedItem.ToString());

            FileInfo[] files = new DirectoryInfo(exTextBox26.Text).GetFiles();
            string fileType = string.Empty;

            progressBar1.Value = 0;
            progressBar1.Maximum = files.Length;
            progressBar1.Step = 1;

            foreach (FileInfo inf in files)
            {
                progressBar1.PerformStep();
                switch (inf.Extension)
                {
                    case ".dta":
                        fileType = "DTA";
                        break;

                    case ".ent":
                        fileType = "ENT";
                        break;

                    case ".cs":
                        fileType = "CS";
                        break;

                    default:
                        fileType = string.Empty;
                        break;
                }

                if (fileType != string.Empty)
                {
                    string name = Path.GetFileNameWithoutExtension(inf.Name);
                    string newFileName = string.Format("{0}\\{1}.{2}", f.SelectedPath, name, outputExt);
                    int count = 2;
                    while (File.Exists(newFileName))
                    {
                        newFileName = string.Format("{0}\\{1}({2}).{3}", f.SelectedPath, name, count, outputExt);
                        count++;
                    }

                    PluginManager.ConvertPlugin(
                        inf.FullName,
                        (PluginManager.PluginType)Enum.Parse(typeof(PluginManager.PluginType), fileType),
                        newFileName,
                        outputType
                        );
                }
            }

            progressBar1.Value = 0;
        }
    }
}