using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Extended;

namespace H3UtilityToolkit.Essentials
{
    /// <summary>
    /// Object for reading, storing, and editing data from the Header.
    /// </summary>
    public class Header
    {
        #region Instance Data

        #region Constants

        /// <summary>
        /// Length of Header in bytes.
        /// </summary>
        public const int Length = 12288;

        #endregion

        protected Map __map__;
        private string __wordHead__;
        private int __version__;
        private int __filesize__;
        private int __indexOffset__;
        private int __virtSegmentStart__;
        private int __virtSegmentSize__;
        private string __buildInfo__;
        private short __mapTypeIndex__;
        private int __stringTableCount__;
        private int __stringTableSize__;
        private int __stringTableIndexOffset__;
        private int __stringTableOffset__;
        private string __internalName__;
        private string __scenarioName__;
        private int __fileTableCount__;
        private int __fileTableOffset__;
        private int __fileTableSize__;
        private int __fileTableIndexOffset__;
        private int __checksum__;
        private int __mapMagicBaseAddr__;
        private byte[] __hash__;
        private int __mapMagicAddrMod1__;
        private int __localeTableAddrMod__;
        private int __mapMagicAddrMod2__;
        private string __wordFoot__;
        private int __headerMagic__;
        private int __mapMagic__;

        #endregion

        #region Accessors

        /// <summary>
        /// Map
        /// </summary>
        protected Map Map { get { return __map__; } set { __map__ = value; } }
        /// <summary>
        /// WordHead
        /// </summary>
        public string WordHead { get { return __wordHead__; } set { __wordHead__ = value; } }
        /// <summary>
        /// Version
        /// </summary>
        public int Version { get { return __version__; } set { __version__ = value; } }
        /// <summary>
        /// Filesize
        /// </summary>
        public int Filesize { get { return __filesize__; } set { __filesize__ = value; } }
        /// <summary>
        /// IndexOffset
        /// </summary>
        public int IndexOffset { get { return __indexOffset__; } set { __indexOffset__ = value; } }
        /// <summary>
        /// VirtSegmentStart
        /// </summary>
        public int VirtSegmentStart { get { return __virtSegmentStart__; } set { __virtSegmentStart__ = value; } }
        /// <summary>
        /// VirtSegmentSize
        /// </summary>
        public int VirtSegmentSize { get { return __virtSegmentSize__; } set { __virtSegmentSize__ = value; } }
        /// <summary>
        /// BuildInfo
        /// </summary>
        public string BuildInfo { get { return __buildInfo__; } set { __buildInfo__ = value; } }
        /// <summary>
        /// MapTypeIndex
        /// </summary>
        public short MapTypeIndex { get { return __mapTypeIndex__; } set { __mapTypeIndex__ = value; } }
        /// <summary>
        /// StringTableCount
        /// </summary>
        public int StringTableCount { get { return __stringTableCount__; } set { __stringTableCount__ = value; } }
        /// <summary>
        /// StringTableSize
        /// </summary>
        public int StringTableSize { get { return __stringTableSize__; } set { __stringTableSize__ = value; } }
        /// <summary>
        /// StringTableIndexOffset
        /// </summary>
        public int StringTableIndexOffset { get { return __stringTableIndexOffset__; } set { __stringTableIndexOffset__ = value; } }
        /// <summary>
        /// StringTableOffset
        /// </summary>
        public int StringTableOffset { get { return __stringTableOffset__; } set { __stringTableOffset__ = value; } }
        /// <summary>
        /// InternalName
        /// </summary>
        public string InternalName { get { return __internalName__; } set { __internalName__ = value; } }
        /// <summary>
        /// ScenarioName
        /// </summary>
        public string ScenarioName { get { return __scenarioName__; } set { __scenarioName__ = value; } }
        /// <summary>
        /// FileTableCount
        /// </summary>
        public int FileTableCount { get { return __fileTableCount__; } set { __fileTableCount__ = value; } }
        /// <summary>
        /// FileTableOffset
        /// </summary>
        public int FileTableOffset { get { return __fileTableOffset__; } set { __fileTableOffset__ = value; } }
        /// <summary>
        /// FileTableSize
        /// </summary>
        public int FileTableSize { get { return __fileTableSize__; } set { __fileTableSize__ = value; } }
        /// <summary>
        /// FileTableIndexOffset
        /// </summary>
        public int FileTableIndexOffset { get { return __fileTableIndexOffset__; } set { __fileTableIndexOffset__ = value; } }
        /// <summary>
        /// Checksum
        /// </summary>
        public int Checksum { get { return __checksum__; } set { __checksum__ = value; } }
        /// <summary>
        /// MapMagicBaseAddr
        /// </summary>
        public int MapMagicBaseAddr { get { return __mapMagicBaseAddr__; } set { __mapMagicBaseAddr__ = value; } }
        /// <summary>
        /// Hash
        /// </summary>
        public byte[] Hash { get { return __hash__; } set { __hash__ = value; } }
        /// <summary>
        /// MapMagicAddrMod1
        /// </summary>
        public int MapMagicAddrMod1 { get { return __mapMagicAddrMod1__; } set { __mapMagicAddrMod1__ = value; } }
        /// <summary>
        /// LocaleTableAddrMod
        /// </summary>
        public int LocaleTableAddrMod { get { return __localeTableAddrMod__; } set { __localeTableAddrMod__ = value; } }
        /// <summary>
        /// MapMagicAddrMod2
        /// </summary>
        public int MapMagicAddrMod2 { get { return __mapMagicAddrMod2__; } set { __mapMagicAddrMod2__ = value; } }
        /// <summary>
        /// WordFoot
        /// </summary>
        public string WordFoot { get { return __wordFoot__; } set { __wordFoot__ = value; } }
        /// <summary>
        /// HeaderMagic
        /// </summary>
        public int HeaderMagic { get { return __headerMagic__; } set { __headerMagic__ = value; } }
        /// <summary>
        /// MapMagic
        /// </summary>
        public int MapMagic { get { return __mapMagic__; } set { __mapMagic__ = value; } }

        #endregion

        /// <summary>
        /// Initialises a new instance of the Header class based on the
        /// supplied Map reference.
        /// </summary>
        /// <param name="map">The supplied Map reference. </param>
        public Header(Map map)
        {
            this.Map = map;
        }

        /// <summary>
        /// Initialises a new instance of the Header class based on the
        /// supplied Map reference, and begins reading straight away with
        /// the supplied BitReader.
        /// </summary>
        /// <param name="map">The supplied Map reference. </param>
        /// <param name="br">The supplied BitReader. </param>
        public Header(Map map, BitReader br)
        {
            this.Map = map;
            this.Read(br);
        }

        /// <summary>
        /// Reads data from the Header and stores it for access later.
        /// </summary>
        public void Read()
        {
            BitReader br = new BitReader(Map.FileName, Endian.Big);
            this.Read(br);
            br.Close();
        }

        /// <summary>
        /// Reads data from the Header and stores it for access later
        /// with the supplied BitReader.
        /// </summary>
        /// <param name="br">The supplied BitReader. </param>
        public void Read(BitReader br)
        {
            br.BaseStream.Position = 0;

            WordHead = br.ReadString(4);
            Version = br.ReadInt32();                // 11 For H3 Maps
            if (Version != 11) throw new Exception("Halo 3 xbox 360 map file expected."); // :3
            Filesize = br.ReadInt32();
            br.BaseStream.Position += 4;
            IndexOffset = br.ReadInt32();            // Address
            VirtSegmentStart = br.ReadInt32();       // Guess
            VirtSegmentSize = br.ReadInt32();        // Guess
            br.BaseStream.Position += 256;
            BuildInfo = br.ReadString(32);
            MapTypeIndex = br.ReadInt16();           // 0=SP 1=MP 2=MM 3=Shared
            br.BaseStream.Position += 26;
            StringTableCount = br.ReadInt32();
            StringTableSize = br.ReadInt32();
            StringTableIndexOffset = br.ReadInt32(); // Address
            StringTableOffset = br.ReadInt32();      // Address
            br.BaseStream.Position += 36;
            InternalName = br.ReadString(32);
            br.BaseStream.Position += 4;
            ScenarioName = br.ReadString(256);
            br.BaseStream.Position += 4;
            FileTableCount = br.ReadInt32();
            FileTableOffset = br.ReadInt32();        // Address
            FileTableSize = br.ReadInt32();
            FileTableIndexOffset = br.ReadInt32();   // Address
            Checksum = br.ReadInt32();               // Every 4 bytes xor'ed together after the header
            br.BaseStream.Position += 32;            // Constant
            MapMagicBaseAddr = br.ReadInt32();
            br.BaseStream.Position += 128;
            Hash = br.ReadBytes(256);
            br.BaseStream.Position += 4;
            MapMagicAddrMod1 = br.ReadInt32();
            br.BaseStream.Position += 4;
            LocaleTableAddrMod = br.ReadInt32();
            br.BaseStream.Position += 12;
            MapMagicAddrMod2 = br.ReadInt32();
            br.BaseStream.Position += 11120;
            WordFoot = br.ReadString(4);

            // Magic

            if (StringTableIndexOffset != 0)
            {
                HeaderMagic = StringTableIndexOffset - Length;
                VirtSegmentStart -= HeaderMagic;
                StringTableIndexOffset -= HeaderMagic;
                StringTableOffset -= HeaderMagic;
                FileTableOffset -= HeaderMagic;
                FileTableIndexOffset -= HeaderMagic;
            }

            MapMagic = MapMagicBaseAddr - (MapMagicAddrMod1 + MapMagicAddrMod2);
            if (IndexOffset != 0) IndexOffset -= MapMagic;
        }

        /// <summary>
        /// Parses the data read from the Header into a ListViewItem array.
        /// </summary>
        public ListViewItem[] ParseForListView()
        {
            List<ListViewItem> items = new List<ListViewItem>();
            int Pos = 0;
            Pos += AddItemToMapHeaderList("Head", WordHead, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Version", Version, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Filesize", Filesize, Pos, 4, items);
            Pos += AddItemToMapHeaderList("NULL", string.Empty, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Index Offset", IndexOffset, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Virt Seg. Start (Guess)", VirtSegmentStart, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Virt Seg. Size (Guess)", VirtSegmentSize, Pos, 4, items);
            Pos += AddItemToMapHeaderList("NULL", string.Empty, Pos, 256, items);
            Pos += AddItemToMapHeaderList("Build Info", BuildInfo, Pos, 32, items);
            Pos += AddItemToMapHeaderList("Map Type Index", MapTypeIndex, Pos, 2, items);
            Pos += AddItemToMapHeaderList("Unk1", string.Empty, Pos, 26, items);
            Pos += AddItemToMapHeaderList("String Table Count", StringTableCount, Pos, 4, items);
            Pos += AddItemToMapHeaderList("String Table Size", StringTableSize, Pos, 4, items);
            Pos += AddItemToMapHeaderList("String Table Index Offset", StringTableIndexOffset, Pos, 4, items);
            Pos += AddItemToMapHeaderList("String Table Offset", StringTableOffset, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Unk2", string.Empty, Pos, 36, items);
            Pos += AddItemToMapHeaderList("Internal Name", InternalName, Pos, 32, items);
            Pos += AddItemToMapHeaderList("NULL", string.Empty, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Scenario Name", ScenarioName, Pos, 256, items);
            Pos += AddItemToMapHeaderList("Unk3", string.Empty, Pos, 4, items);
            Pos += AddItemToMapHeaderList("File Table Count", FileTableCount, Pos, 4, items);
            Pos += AddItemToMapHeaderList("File Table Offset", FileTableOffset, Pos, 4, items);
            Pos += AddItemToMapHeaderList("File Table Size", FileTableSize, Pos, 4, items);
            Pos += AddItemToMapHeaderList("File Table Index Offset", FileTableIndexOffset, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Checksum", Checksum, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Unk4 - Constant", string.Empty, Pos, 32, items);
            Pos += AddItemToMapHeaderList("Map Magic Base Addr.", MapMagicBaseAddr, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Unk5", string.Empty, Pos, 128, items);
            Pos += AddItemToMapHeaderList("Hash", string.Empty, Pos, 256, items);
            Pos += AddItemToMapHeaderList("Unk6", string.Empty, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Map Magic Addr. Mod1", MapMagicAddrMod1, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Unk7", string.Empty, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Locale Table Addr. Mod", LocaleTableAddrMod, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Unk8", string.Empty, Pos, 12, items);
            Pos += AddItemToMapHeaderList("Map Magic Addr. Mod2", MapMagicAddrMod2, Pos, 4, items);
            Pos += AddItemToMapHeaderList("Unk9", string.Empty, Pos, 11120, items);
            Pos += AddItemToMapHeaderList("Foot", WordFoot, Pos, 4, items);
            AddItemToMapHeaderList("Header Magic", HeaderMagic, -1, 4, items);
            AddItemToMapHeaderList("Map Magic", MapMagic, -1, 4, items);
            return items.ToArray();
        }

        private int AddItemToMapHeaderList(object arg, object val, int pos, int len, List<ListViewItem> lvList)
        {
            ListViewItem item = new ListViewItem(arg.ToString());
            item.SubItems.Add(val == null ? string.Empty : val.ToString());
            item.SubItems.Add(pos.ToString());
            item.SubItems.Add(len.ToString());
            lvList.Add(item);
            return len;
        }

        /// <summary>
        /// Writes data stored in the class to the Header.
        /// </summary>
        public void Write()
        {
            BitWriter bw = new BitWriter(Map.FileName, Endian.Big);
            this.Write(bw);
            bw.Close();
        }

        /// <summary>
        /// Writes data stored in the class to the Header with
        /// the supplied BitWriter.
        /// </summary>
        /// <param name="bw">The supplied BitWriter. </param>
        public void Write(BitWriter bw)
        {
            bw.BaseStream.Position = 0;

            // TODO
        }
    }

    /// <summary>
    /// The Type of Map.
    /// </summary>
    public enum MapType
    {
        Single_Player_Map,
        Multi_Player_Map,
        MainMenu,
        Shared
    }
}