using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Extended;

namespace H3UtilityToolkit.Essentials
{
    /// <summary>
    /// Object for reading, storing, and editing data from the Index.
    /// </summary>
    public class Index
    {
        #region Instance Data

        #region Constants

        /// <summary>
        /// Length of Index in bytes.
        /// </summary>
        public const int Length = 40;

        #endregion

        protected Map __map__;
        private int __tagClassCount__;
        private int __tagClassIndexOffset__;
        private int __tagCount__;
        private int __tagInfoOffset__;
        private int __tagInfoHeaderCount__;
        private int __tagInfoHeaderOffset__;
        private int __tagInfoHeaderCount2__;
        private int __tagInfoHeaderOffset2__;
        private int __unk1__;
        private string __wordTags__;

        #endregion

        #region Accessors

        /// <summary>
        /// Map
        /// </summary>
        protected Map Map { get { return __map__; } set { __map__ = value; } }
        /// <summary>
        /// TagClassCount
        /// </summary>
        public int TagClassCount { get { return __tagClassCount__; } set { __tagClassCount__ = value; } }
        /// <summary>
        /// TagClassIndexOffset
        /// </summary>
        public int TagClassIndexOffset { get { return __tagClassIndexOffset__; } set { __tagClassIndexOffset__ = value; } }
        /// <summary>
        /// TagCount
        /// </summary>
        public int TagCount { get { return __tagCount__; } set { __tagCount__ = value; } }
        /// <summary>
        /// TagInfoOffset
        /// </summary>
        public int TagInfoOffset { get { return __tagInfoOffset__; } set { __tagInfoOffset__ = value; } }
        /// <summary>
        /// TagInfoHeaderCount
        /// </summary>
        public int TagInfoHeaderCount { get { return __tagInfoHeaderCount__; } set { __tagInfoHeaderCount__ = value; } }
        /// <summary>
        /// TagInfoHeaderOffset
        /// </summary>
        public int TagInfoHeaderOffset { get { return __tagInfoHeaderOffset__; } set { __tagInfoHeaderOffset__ = value; } }
        /// <summary>
        /// TagInfoHeaderCount2
        /// </summary>
        public int TagInfoHeaderCount2 { get { return __tagInfoHeaderCount2__; } set { __tagInfoHeaderCount2__ = value; } }
        /// <summary>
        /// TagInfoHeaderOffset2
        /// </summary>
        public int TagInfoHeaderOffset2 { get { return __tagInfoHeaderOffset2__; } set { __tagInfoHeaderOffset2__ = value; } }
        /// <summary>
        /// Unk1
        /// </summary>
        public int Unk1 { get { return __unk1__; } set { __unk1__ = value; } }
        /// <summary>
        /// WordTags
        /// </summary>
        public string WordTags { get { return __wordTags__; } set { __wordTags__ = value; } }

        #endregion

        /// <summary>
        /// Initialises a new instance of the Index class based on the
        /// supplied Map reference.
        /// </summary>
        /// <param name="map">The supplied Map reference. </param>
        public Index(Map map)
        {
            this.Map = map;
        }

        /// <summary>
        /// Initialises a new instance of the Index class based on the
        /// supplied Map reference, and begins reading straight away with
        /// the supplied BitReader.
        /// </summary>
        /// <param name="map">The supplied Map reference. </param>
        /// <param name="br">The supplied BitReader. </param>
        public Index(Map map, BitReader br)
        {
            this.Map = map;
            this.Read(br);
        }

        /// <summary>
        /// Reads data from the Index and stores it for access later.
        /// </summary>
        public void Read()
        {
            BitReader br = new BitReader(Map.FileName, Endian.Big);
            this.Read(br);
            br.Close();
        }

        /// <summary>
        /// Reads data from the Index and stores it for access later
        /// with the supplied BitReader.
        /// </summary>
        /// <param name="br">The supplied BitReader. </param>
        public void Read(BitReader br)
        {
            if (Map.Header.IndexOffset < Header.Length ||
                Map.Header.IndexOffset > br.BaseStream.Length) return;

            br.BaseStream.Position = Map.Header.IndexOffset;

            TagClassCount = br.ReadInt32();
            TagClassIndexOffset = br.ReadInt32();  // Address
            TagCount = br.ReadInt32();
            TagInfoOffset = br.ReadInt32();        // Address
            TagInfoHeaderCount = br.ReadInt32();
            TagInfoHeaderOffset = br.ReadInt32();  // Address
            TagInfoHeaderCount2 = br.ReadInt32();
            TagInfoHeaderOffset2 = br.ReadInt32(); // Address
            Unk1 = br.ReadInt32();
            WordTags = br.ReadString(4);

            TagClassIndexOffset -= Map.Header.MapMagic;
            TagInfoOffset -= Map.Header.MapMagic;
            TagInfoHeaderOffset -= Map.Header.MapMagic;
            TagInfoHeaderOffset2 -= Map.Header.MapMagic;
        }

        /// <summary>
        /// Parses the data read from the Index into a ListViewItem array.
        /// </summary>
        public ListViewItem[] ParseForListView()
        {
            List<ListViewItem> items = new List<ListViewItem>();
            int Pos = Map.Header.IndexOffset;
            Pos += AddItemToMapIndexList("TagClassCount", TagClassCount, Pos, 4, items);
            Pos += AddItemToMapIndexList("TagClassIndexOffset", TagClassIndexOffset, Pos, 4, items);
            Pos += AddItemToMapIndexList("TagCount", TagCount, Pos, 4, items);
            Pos += AddItemToMapIndexList("TagInfoOffset", TagInfoOffset, Pos, 4, items);
            Pos += AddItemToMapIndexList("TagInfoHeaderCount", TagInfoHeaderCount, Pos, 4, items);
            Pos += AddItemToMapIndexList("TagInfoHeaderOffset", TagInfoHeaderOffset, Pos, 4, items);
            Pos += AddItemToMapIndexList("TagInfoHeaderCount2", TagInfoHeaderCount2, Pos, 4, items);
            Pos += AddItemToMapIndexList("TagInfoHeaderOffset2", TagInfoHeaderOffset2, Pos, 4, items);
            Pos += AddItemToMapIndexList("Unk1", Unk1, Pos, 4, items);
            Pos += AddItemToMapIndexList("WordTags", WordTags, Pos, 4, items);
            return items.ToArray();
        }

        private int AddItemToMapIndexList(object arg, object val, int pos, int len, List<ListViewItem> lvList)
        {
            ListViewItem item = new ListViewItem(arg.ToString());
            item.SubItems.Add(val == null ? string.Empty : val.ToString());
            item.SubItems.Add(pos.ToString());
            item.SubItems.Add(len.ToString());
            lvList.Add(item);
            return len;
        }

        /// <summary>
        /// Writes data stored in the class to the Index.
        /// </summary>
        public void Write()
        {
            BitWriter bw = new BitWriter(Map.FileName, Endian.Big);
            this.Write(bw);
            bw.Close();
        }

        /// <summary>
        /// Writes data stored in the class to the Index with
        /// the supplied BitWriter.
        /// </summary>
        /// <param name="bw">The supplied BitWriter. </param>
        public void Write(BitWriter bw)
        {
            bw.BaseStream.Position = Map.Header.IndexOffset;

            // TODO
        }
    }
}