using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Extended;
using H3UtilityToolkit.Essentials;

namespace H3UtilityToolkit.Misc
{
    /// <summary>
    /// Misc funtions, map related.
    /// </summary>
    public class MiscUtilities
    {
        #region Instance Data

        private Map __map__;

        #endregion

        #region Accessors

        /// <summary>
        /// Map
        /// </summary>
        public Map Map { get { return __map__; } set { __map__ = value; } }

        #endregion

        /// <summary>
        /// Initialises a new instance of the MiscUtilities class based on the
        /// supplied Map reference.
        /// </summary>
        /// <param name="map">The supplied Map reference. </param>
        public MiscUtilities(Map map)
        {
            this.Map = map;
        }

        public string GetMapNameFromAssetLocation(AssetLocation assetLoc)
        {
            switch (assetLoc)
            {
                case AssetLocation.Internal:
                    return Path.GetFileNameWithoutExtension(Map.FileName);

                case AssetLocation.MainMenu:
                    return "mainmenu";

                case AssetLocation.MPShared:
                    return "shared";

                case AssetLocation.SPShared:
                    return "campaign";

                default:
                    return string.Empty;
            }
        }

        public static string RemoveIllegalFileNameChars(string fileName)
        {
            StringBuilder sb = new StringBuilder(fileName);
            char[] invalid = Path.GetInvalidFileNameChars();

            for (int x = 0; x < sb.Length; x++)
            {
                for (int y = 0; y < invalid.Length; y++)
                {
                    if (sb[x] == invalid[y])
                        sb[x] = '_';
                }
            }

            return sb.ToString();
        }

        public static string CapitaliseLetter(int index, string str)
        {
            if (str.Length == 0) return string.Empty;

            if (index == 0)
            {
                return str.Substring(0, 1).ToUpper() + str.Substring(1);
            }
            else if (index == str.Length - 1)
            {
                return str.Substring(0, index) + str.Substring(index, 1).ToUpper();
            }
            else
            {
                return str.Substring(0, index) + str.Substring(index, 1).ToUpper() + str.Substring(index + 1);
            }
        }

        public static string LowerCaseLetter(int index, string str)
        {
            if (str.Length == 0) return string.Empty;

            if (index == 0)
            {
                return str.Substring(0, 1).ToLower() + str.Substring(1);
            }
            else if (index == str.Length - 1)
            {
                return str.Substring(0, index) + str.Substring(index, 1).ToLower();
            }
            else
            {
                return str.Substring(0, index) + str.Substring(index, 1).ToLower() + str.Substring(index + 1);
            }
        }
    }

    /// <summary>
    /// The Location of the Asset.
    /// </summary>
    public enum AssetLocation
    {
        Internal,
        MainMenu,
        MPShared,
        SPShared
    }

    /// <summary>
    /// Designed to be used when no other sorting
    /// alorithm is available. [Warning: Boxing will occur!]
    /// </summary>
    public class H3TagSorter : System.Collections.IComparer
    {
        public int Compare(object x, object y)
        {
            return ((Tag)x).Path.CompareTo(((Tag)y).Path);
        }
    }
}