using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace H3UtilityToolkit.PluginStuffs
{
    /// <summary>
    /// 
    /// </summary>
    public partial class MetaEditorPanel : UserControl
    {
        #region Accessors

        /// <summary>
        /// Gets the width of the control area.
        /// </summary>
        public int EditorWidth { get { return panel1.Width; } }
        /// <summary>
        /// Gets or sets the background color for the control.
        /// </summary>
        public Color EditorBackColor { get { return panel1.BackColor; } set { panel1.BackColor = value; this.BackColor = value; } }

        #endregion

        #region EventHandlers

        public event EventHandler OnResizeHandler;

        #endregion

        /// <summary>
        /// 
        /// </summary>
        public MetaEditorPanel()
        {
            InitializeComponent();
            vScrollBar1.SmallChange = 5;
        }

        #region Controls

        /// <summary>
        /// Adds an array of control objects to the collection.
        /// </summary>
        /// <param name="controls">An array of System.Windows.Forms.Control objects to add to the collection. </param>
        public void AddControlRange(Control[] controls)
        {
            panel1.Controls.AddRange(controls);
            panel1.Height = panel1.PreferredSize.Height;
            UpdateScrollBar();
        }

        /// <summary>
        /// Removes all controls from the collection.
        /// </summary>
        public void ClearControls()
        {
            panel1.Controls.Clear();
            vScrollBar1.Value = 0;
            vScrollBar1.Enabled = false;
        }

        #endregion

        #region ScrollBar

        private void UpdateScrollBar()
        {
            // No controls || no scrollbar needed
            if (panel1.Controls.Count == 0 || (panel1.Height - this.Height) <= 0)
            {
                vScrollBar1.Enabled = false;
                vScrollBar1.Value = 0;
                vScrollBar1.LargeChange = 0;
                vScrollBar1.Maximum = 100;
            }
            else
            {
                vScrollBar1.Enabled = true;
                if (ValueIsMax()) SetValueToNewMax(this.Height, panel1.Height);
                vScrollBar1.Maximum = panel1.Height;
                vScrollBar1.LargeChange = this.Height;
            }

            ScrollEventArgs sea = new ScrollEventArgs(ScrollEventType.ThumbPosition, vScrollBar1.Value);
            vScrollBar1_Scroll(null, sea);
        }

        private bool ValueIsMax()
        {
            return (vScrollBar1.Value + vScrollBar1.LargeChange >= vScrollBar1.Maximum);
        }

        private void SetValueToNewMax(int newLargeChange, int newMax)
        {
            vScrollBar1.Value = newMax - newLargeChange;
        }

        private void vScrollBar1_Scroll(object sender, ScrollEventArgs e)
        {
            panel1.Top = -e.NewValue;
        }

        #endregion

        #region Resize

        private void MetaEditorPanel_Resize(object sender, EventArgs e)
        {
            panel1.Width = this.Width - vScrollBar1.Width;
            if (OnResizeHandler != null) OnResizeHandler(panel1, EventArgs.Empty);
            UpdateScrollBar();
        }

        #endregion
    }
}
