using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Johnson;

namespace H3UtilityToolkit.PluginStuffs
{
    public partial class MetaEditorPrefs : Form
    {
        MapForm mf;
        bool bPage1, bPage2;

        public MetaEditorPrefs(MapForm mf)
        {
            InitializeComponent();
            this.mf = mf;

            string[] dataTypes = Enum.GetNames(typeof(NodeType));
            for (int x = 0; x < dataTypes.Length; x++)
            {
                NodeType nt = (NodeType)Enum.Parse(typeof(NodeType), dataTypes[x]);
                //if ((NodeType.All & nt) == nt)
                if (nt != NodeType.None && nt != NodeType.All && nt != NodeType.BoxUsers)
                {
                    checkedListBox1.Items.Add(dataTypes[x], PluginManager.IsVisible(nt));
                }
            }

            panel1.BackColor = PluginManager.ReflexiveBaseColour;
            trackBar1.Value = PluginManager.ReflexiveRedShift;
            trackBar2.Value = PluginManager.ReflexiveGreenShift;
            trackBar3.Value = PluginManager.ReflexiveBlueShift;

            UpdatePreviewPanels();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            for (int x = 0; x < checkedListBox1.Items.Count; x++)
                checkedListBox1.SetItemChecked(x, true);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            for (int x = 0; x < checkedListBox1.Items.Count; x++)
                checkedListBox1.SetItemChecked(x, false);
        }

        private void checkedListBox1_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            bool eItemChecked = (e.NewValue == CheckState.Checked ? true : false);
            for (int x = 0; x < checkedListBox1.Items.Count; x++)
            {
                NodeType nt = (NodeType)Enum.Parse(typeof(NodeType), checkedListBox1.Items[x].ToString());

                bool xItemChecked = (x == e.Index ? eItemChecked : checkedListBox1.GetItemChecked(x));
                if (xItemChecked != PluginManager.IsVisible(nt))
                {
                    button4.Enabled = true;
                    bPage1 = true;
                    return;
                }
            }

            if (!bPage2) button4.Enabled = false;
            bPage1 = false;
        }

        private void panel1_Click(object sender, EventArgs e)
        {
            cd1.Color = panel1.BackColor;
            if (cd1.ShowDialog() == DialogResult.Cancel) return;
            panel1.BackColor = cd1.Color;

            UpdatePreviewPanels();
        }

        private void trackBar123_Scroll(object sender, EventArgs e)
        {
            UpdatePreviewPanels();
        }

        public void UpdatePreviewPanels()
        {
            Color clr = panel2.BackColor = panel1.BackColor;
            for (int x = 3; x < 22; x++)
            {
                //update clr
                int tmpR = clr.R + trackBar1.Value;
                tmpR = tmpR > 255 ? panel2.BackColor.R : tmpR;

                int tmpG = clr.G + trackBar2.Value;
                tmpG = tmpG > 255 ? panel2.BackColor.G : tmpG;

                int tmpB = clr.B + trackBar3.Value;
                tmpB = tmpB > 255 ? panel2.BackColor.B : tmpB;

                clr = Color.FromArgb(tmpR, tmpG, tmpB);

                //set clr
                tabPage2.Controls["panel" + x.ToString()].BackColor = clr;
            }

            if (panel1.BackColor == PluginManager.ReflexiveBaseColour &&
                trackBar1.Value == PluginManager.ReflexiveRedShift &&
                trackBar2.Value == PluginManager.ReflexiveGreenShift &&
                trackBar3.Value == PluginManager.ReflexiveBlueShift)
            {
                if (!bPage1) button4.Enabled = false;
                bPage2 = false;
            }
            else
            {
                button4.Enabled = true;
                bPage2 = true;
            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            // Visible Datatypes.
            PluginManager.VisibleMask = NodeType.None;
            for (int x = 0; x < checkedListBox1.Items.Count; x++)
            {
                if (checkedListBox1.GetItemChecked(x))
                {
                    NodeType nt = (NodeType)Enum.Parse(typeof(NodeType), checkedListBox1.Items[x].ToString());
                    PluginManager.VisibleMask |= nt;
                }
            }

            // Reflexives.
            PluginManager.ReflexiveBaseColour = mf.metaEditorPanel1.EditorBackColor = panel1.BackColor;
            PluginManager.ReflexiveRedShift = trackBar1.Value;
            PluginManager.ReflexiveGreenShift = trackBar2.Value;
            PluginManager.ReflexiveBlueShift = trackBar3.Value;

            if (bPage1)
            {
                mf.metaEditorPanel1.ClearControls();
                mf.metaEditorPanel1.AddControlRange(mf.map.PluginManager.ParseForViewing(mf.metaEditorPanel1.EditorWidth));
            }
            else if (bPage2 && PluginManager.IsVisible(NodeType.Reflexive))
            {
                for (int x = 0; x < mf.map.PluginManager.CurrentPlugin.Nodes.Count; x++)
                {
                    if (mf.map.PluginManager.CurrentPlugin.Nodes[x].NodeType == NodeType.Reflexive)
                    {
                        Color baseClr = (mf.map.PluginManager.CurrentPlugin.Nodes[x] as StructNode).Parent == null ?
                            PluginManager.ReflexiveBaseColour
                            : (mf.map.PluginManager.CurrentPlugin.Nodes[x] as StructNode).Parent.Clr;

                        int tmpR = baseClr.R + PluginManager.ReflexiveRedShift;
                        if (tmpR > 255) tmpR = PluginManager.ReflexiveBaseColour.R;
                        int tmpG = baseClr.G + PluginManager.ReflexiveGreenShift;
                        if (tmpG > 255) tmpG = PluginManager.ReflexiveBaseColour.G;
                        int tmpB = baseClr.B + PluginManager.ReflexiveBlueShift;
                        if (tmpB > 255) tmpB = PluginManager.ReflexiveBaseColour.B;

                        (mf.map.PluginManager.CurrentPlugin.Nodes[x] as StructNode).Clr =
                        (mf.map.PluginManager.CurrentPlugin.Nodes[x] as StructNode).Panel1.BackColor =
                        Color.FromArgb(tmpR, tmpG, tmpB);
                    }
                }
            }
            this.Close();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}