using System;
using System.Windows.Forms;
using System.Collections.Generic;
using System.IO;
using System.Extended;
using H3UtilityToolkit;

namespace H3UtilityToolkit.Structures
{
    /// <summary>
    /// Global
    /// </summary>
    public class Matg
    {
        #region Instance Data

        private Map __map__;
        private int __identity__;
        private Locale __selectedLocale__;
        private int __sTableCount__;
        private int __sTableSize__;
        private int __sTableIndexOffset__;
        private int __sTableOffset__;
        private int[] __sLocaleOffsets__;
        private string[] __sLocaleStrings__;

        #endregion

        #region Accessors

        /// <summary>
        /// Map reference, used when in need of other functions outside of this class.
        /// </summary>
        public Map Map { get { return __map__; } set { __map__ = value; } }
        /// <summary>
        /// The identity of the tag to be used by this class.
        /// </summary>
        public int Identity { get { return __identity__; } set { __identity__ = value; } }
        /// <summary>
        /// SelectedLocale
        /// </summary>
        public Locale SelectedLocale { get { return __selectedLocale__; } set { __selectedLocale__ = value; } }
        /// <summary>
        /// TableCount
        /// </summary>
        public int TableCount { get { return __sTableCount__; } set { __sTableCount__ = value; } }
        /// <summary>
        /// TableSize
        /// </summary>
        public int TableSize { get { return __sTableSize__; } set { __sTableSize__ = value; } }
        /// <summary>
        /// TableIndexOffset
        /// </summary>
        public int TableIndexOffset { get { return __sTableIndexOffset__; } set { __sTableIndexOffset__ = value; } }
        /// <summary>
        /// TableOffset
        /// </summary>
        public int TableOffset { get { return __sTableOffset__; } set { __sTableOffset__ = value; } }
        /// <summary>
        /// LocaleOffsets
        /// </summary>
        public int[] LocaleOffsets { get { return __sLocaleOffsets__; } set { __sLocaleOffsets__ = value; } }
        /// <summary>
        /// LocaleStrings
        /// </summary>
        public string[] LocaleStrings { get { return __sLocaleStrings__; } set { __sLocaleStrings__ = value; } }

        #endregion

        /// <summary>
        /// Initializes a new instance of the H3UtilityToolkit.Structures.Matg class based 
        /// on the supplied map reference and tag identity.
        /// </summary>
        /// <param name="map">The map reference. </param>
        /// <param name="identity">The identity of the tag to be used by this class. </param>
        public Matg(Map map, int identity)
        {
            this.Map = map;
            this.Identity = identity;
        }

        #region [matg] Related..

        public void LoadLocaleTable(Locale loc)
        {
            BitReader br = new BitReader(Map.FileName, Endian.Big);
            LoadLocaleTable(br, loc);
            br.Close();
        }

        public void LoadLocaleTable(BitReader br, Locale loc)
        {
            int tagIndx = Map.Tags.IndexOf(Identity);
            if (tagIndx == -1) return;

            br.BaseStream.Position = Map.Tags.TagsList[tagIndx].MetaOffset
                + 452 + ((int)loc * 68);

            TableCount = br.ReadInt32();
            TableSize = br.ReadInt32();
            TableIndexOffset = br.ReadInt32(); // Address
            TableOffset = br.ReadInt32();      // Address

            TableIndexOffset += Map.Header.LocaleTableAddrMod;
            TableOffset += Map.Header.LocaleTableAddrMod;

            // Gather the offsets
            LocaleOffsets = new int[TableCount];
            if (TableCount > 0)
            {
                br.BaseStream.Position = TableIndexOffset;
                for (int x = 0; x < TableCount; x++)
                {
                    br.BaseStream.Position += 4;
                    LocaleOffsets[x] = br.ReadInt32();
                }
            }

            // Gather the strings
            LocaleStrings = new string[TableCount];
            if (TableCount > 0)
            {
                int indx;
                for (indx = 0; indx < TableCount - 1; indx++)
                {
                    br.BaseStream.Position = TableOffset + LocaleOffsets[indx];
                    LocaleStrings[indx] = br.ReadString(LocaleOffsets[indx + 1] - LocaleOffsets[indx] - 1);
                }
                br.BaseStream.Position = TableOffset + LocaleOffsets[indx];
                LocaleStrings[indx] = br.ReadString(TableSize - LocaleOffsets[indx] - 1);
            }

            SelectedLocale = loc;
        }

        public ListViewItem[] ParseForListview()
        {
            // Gather the strings
            ListViewItem[] items = new ListViewItem[TableCount];
            for (int x = 0; x < TableCount; x++)
            {
                items[x] = new ListViewItem(LocaleStrings[x]);
                items[x].SubItems.Add(LocaleOffsets[x].ToString());
                items[x].SubItems.Add(LocaleStrings[x].Length.ToString());
            }
            return items;
        }

        public void Write(int index, string newString)
        {
            BitWriter bw = new BitWriter(Map.FileName, Endian.Big);
            Write(bw, index, newString);
            bw.Close();
        }

        public void Write(BitWriter bw, int index, string newString)
        {
            bw.BaseStream.Position = TableOffset + LocaleOffsets[index];

            if (newString.Length == LocaleStrings[index].Length)
            {
                bw.Write(newString);
            }
            else if (newString.Length < LocaleStrings[index].Length)
            {
                while (newString.Length != LocaleStrings[index].Length)
                    newString += ' ';

                bw.Write(newString);
            }
            else if (newString.Length > LocaleStrings[index].Length)
            {
                bw.Write(newString, LocaleStrings[index].Length);
            }

            LocaleStrings[index] = newString;
        }

        public enum Locale
        {
            English,
            Japanese,
            German,
            French,
            Spanish,
            LatinAmericanSpanish,
            Italian,
            Korean,
            Chinese,
            Unk1,
            Portuguese,
            Unk2
        }

        #endregion
    }
}